# Generated by roxygen2: do not edit by hand

export(constraint_to_model)
export(model_to_constraint)
export(pgmm_rjmcmc)
export(pgmm_rjmcmc_chains)
export(summarize_pgmm_rjmcmc)
import(MASS)
import(fabMix)
import(label.switching)
import(mcmcse)
import(methods)
import(pgmm)
import(stats)
importFrom(Rcpp,sourceCpp)
importFrom(gtools,rdirichlet)
importFrom(mclust,adjustedRandIndex)
importFrom(mvtnorm,rmvnorm)
useDynLib(bpgmm, .registration = TRUE)
