# # R pkg
#
# library("tidyverse")
# library("fabMix")
# library("mclust")
# library("pgmm")
# library("mvtnorm")
# library("mcmcse")
# library("MASS")
# library("gtools")
# library("RcppArmadillo")
# library("Rcpp")
#
# # R source
#
# source("R/summerizeRes.R")
# source("R/choosem/split.R")
# source("R/choosem/combine.R")
# source("R/pgmmStay.R")
# source("R/pgmmMstep.R")
# source("R/pgmmVstep.R")
# source("R/PGMMmain.R")
# source("R/prior.R")
# source("R/update_post_theta_y.R")
# source("R/utils.R")
# source("R/posteriorZ.R")
# source("R/proposalLambda.R")
# source("R/proposalPsi.R")
# source("R/hyperParamClass.R")
# source("R/paramClass.R")
# source("R/choosem/proposalFunctions.R")
#
# # C++ source
#
# sourceCpp("src/utils.cpp")
# sourceCpp("src/update_post_z.cpp")
# sourceCpp("src/calculate_cxy.cpp")
# sourceCpp("src/calculate_post_lambda_psi.cpp")
# sourceCpp("src/update_latent_scores.cpp")
# sourceCpp("src/update_hyperparameter.cpp")
#
#
#
#
#
