# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calculate_cxy_native <- function(m, n, hparam, theta_y_list, z, q_vec, X) {
    .Call(`_bpgmm_calculate_cxy_native`, m, n, hparam, theta_y_list, z, q_vec, X)
}

calculate_post_lambda_psi_native <- function(m, p, hparam, cxy_list, theta_y_list, q_vec, constraint) {
    .Call(`_bpgmm_calculate_post_lambda_psi_native`, m, p, hparam, cxy_list, theta_y_list, q_vec, constraint)
}

evaluate_prior_psi_native <- function(psy, p, m, delta, bbeta, constraint, clus_ind) {
    .Call(`_bpgmm_evaluate_prior_psi_native`, psy, p, m, delta, bbeta, constraint, clus_ind)
}

evaluate_prior_lambda_native <- function(p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind) {
    .Call(`_bpgmm_evaluate_prior_lambda_native`, p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind)
}

update_latent_scores_native <- function(X, theta_y_list, z, clus_ind, q_vec) {
    .Call(`_bpgmm_update_latent_scores_native`, X, theta_y_list, z, clus_ind, q_vec)
}

update_post_z_native <- function(X, m, n, theta_y_list) {
    .Call(`_bpgmm_update_post_z_native`, X, m, n, theta_y_list)
}

update_hyperparameter_native <- function(m, p, q, hparam, theta_y_list, d_vec, s_vec) {
    .Call(`_bpgmm_update_hyperparameter_native`, m, p, q, hparam, theta_y_list, d_vec, s_vec)
}

get_z_matrix_native <- function(z, m, n) {
    .Call(`_bpgmm_get_z_matrix_native`, z, m, n)
}

multivariate_normal_density_native <- function(x, mean, sigma, logd) {
    .Call(`_bpgmm_multivariate_normal_density_native`, x, mean, sigma, logd)
}

calculate_ratio_native <- function(log_denominator, log_numerator) {
    .Call(`_bpgmm_calculate_ratio_native`, log_denominator, log_numerator)
}

