% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constraints.R
\name{model_to_constraint}
\alias{model_to_constraint}
\title{Convert PGMM Paper Model Names to Constraint Codes}
\usage{
model_to_constraint(model)
}
\arguments{
\item{model}{Character scalar naming one of the eight PGMM covariance
structures: `CCC`, `CCU`, `CUC`, `CUU`, `UCC`, `UCU`, `UUC`, or `UUU`.}
}
\value{
Integer vector of length three. `1` means constrained and `0` means
  unconstrained.
}
\description{
Convert PGMM Paper Model Names to Constraint Codes
}
